package fr.edf.distribution.brms.issue.model.test;

import java.util.Collection;

import org.drools.compiler.kie.builder.impl.InternalKieModule;
import org.drools.core.definitions.impl.KnowledgePackageImpl;
import org.drools.core.impl.KnowledgeBaseImpl;
import org.drools.core.impl.StatefulKnowledgeSessionImpl;
import org.junit.Assert;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieModule;
import org.kie.api.builder.ReleaseId;
import org.kie.api.definition.rule.Rule;
import org.kie.api.io.Resource;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.internal.definition.KnowledgePackage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class IncrementalCompilationTest {

	private static final Logger LOG = LoggerFactory.getLogger(IncrementalCompilationTest.class);

	public static final String RULE_1 = "package fr.edf.distribution.brms.common.test\n" +
					"import fr.edf.distribution.brms.issue.model.DummyEvent\n" +
					"import fr.edf.distribution.brms.issue.model.OtherDummyEvent\n" +
					"rule \"RG_TEST_1\"\n" +
					"    when\n" +
					"       $event: DummyEvent ()\n" +
					"		(or OtherDummyEvent (id == $event.id, dummyField == 'dummy')\n" +
					"       not OtherDummyEvent (id == $event.id))\n" +
					"    then\n" +
					"        insert(new DummyEvent());\n" +
					"end\n";

	@Test
	public void testUpdateRuleWithSubNetwork() throws Exception {
		KieServices ks = KieServices.Factory.get();

		ReleaseId releaseId1 = ks.newReleaseId( "org.kie", "test-upgrade", "1" );
		KieModule km = createAndDeployJar( ks, releaseId1, RULE_1);
		KieContainer kc = ks.newKieContainer( km.getReleaseId() );

		KieBase kieBase = kc.newKieBase(null);
		KieSession kieSession = kieBase.newKieSession();
		((KnowledgeBaseImpl) kieBase).addStatefulSession(((StatefulKnowledgeSessionImpl) kieSession));

		removeRule("RG_TEST_1", kieBase);

	}

	private KieModule createAndDeployJar(KieServices ks, ReleaseId releaseId, String... drls) {
		byte[] jar = createJar(ks, releaseId, drls);
		// Deploy jar into the repository
		KieModule km = deployJarIntoRepository(ks, jar);
		return km;
	}


	private KieModule deployJarIntoRepository(KieServices ks, byte[] jar) {
		Resource jarRes = ks.getResources().newByteArrayResource(jar);
		KieModule km = ks.getRepository().addKieModule(jarRes);
		return km;
	}

	private byte[] createJar(KieServices ks, ReleaseId releaseId, String... drls) {
		KieFileSystem kfs = ks.newKieFileSystem().generateAndWritePomXML(releaseId);
		for (int i = 0; i < drls.length; i++) {
			if (drls[i] != null) {
				kfs.write("src/main/resources/r" + i + ".drl", drls[i]);
			}
		}
		KieBuilder kb = ks.newKieBuilder(kfs).buildAll();
		Assert.assertFalse(kb.getResults().getMessages(org.kie.api.builder.Message.Level.ERROR).toString(),
				kb.getResults().hasMessages(org.kie.api.builder.Message.Level.ERROR));
		InternalKieModule kieModule = (InternalKieModule) ks.getRepository()
				.getKieModule(releaseId);
		byte[] jar = kieModule.getBytes();
		return jar;
	}

	private void removeRule(final String ruleName, KieBase kieBase) {
		Collection<KnowledgePackage> pcks = ((KnowledgeBaseImpl) kieBase).getKnowledgePackages();
		for (KnowledgePackage p : pcks) {
			Rule rule = ((KnowledgePackageImpl) p).getRule(ruleName);
			if (rule != null) {
				try {
					kieBase.removeRule(p.getName(), rule.getName());
				} catch (RuntimeException e) {
					LOG.error("Impossible de supprimer la regle {}", ruleName, e);
					throw e;
				}
				LOG.info("La regle '{}' a ete supprime du package '{}'", rule.getName(), p.getName());
				return;
			}
		}
		LOG.info("La regle '{}' n'existe pas", ruleName);
	}
}
